#include "X_routines.h"

#include <X11/Xlib.h>
#include <X11/keysym.h>
#include <X11/Xutil.h>

#include "evenement.h"
#include "jeu.h"
#include "ecran.h"

Display *X_ecran;
Window X_win;
GC gcontext;
XGCValues gcv;
XFontStruct *X_font;

#define FONT "10x20"

/* que le clavier for the time */
void X_next_evenement(evenement *x)
{
  XEvent X_event;
  char buffer[15];
  KeySym key;
  XComposeStatus compose;
  while (1) {
    do {
      XNextEvent (X_ecran, &X_event);
      if (X_event.type==Expose) affiche_jeu();
    } while (X_event.type!=KeyPress /* ou souris, pas gere encore */);

    if (X_event.type==KeyPress) {
      x->type=CLAVIER;
      XLookupString((XKeyEvent *)&X_event, buffer, 10, &key, &compose);
      switch (key) {
        case XK_Escape : x->cas.touche=ECHAP;         return;
        case XK_Left   : x->cas.touche=FLECHE_GAUCHE; return;
        case XK_Right  : x->cas.touche=FLECHE_DROITE; return;
        case XK_space  : x->cas.touche=ESPACE;        return;
      }
    }
  }
}

#include <stdio.h>
#include <stdlib.h>

void X_init_screen(void)
{
  X_ecran=XOpenDisplay(0);
  if (X_ecran==NULL) {fprintf (stderr, "Impossible ouvrir ecran...\n"); exit(1);}
  X_win=XCreateSimpleWindow (X_ecran, DefaultRootWindow(X_ecran), 0, 0,
			     ECRAN_LARG, ECRAN_HAUT, 6,
			     WhitePixel(X_ecran, DefaultScreen(X_ecran)),
			     BlackPixel(X_ecran, DefaultScreen(X_ecran)));
  XSelectInput (X_ecran, X_win, ExposureMask | KeyPressMask);
  gcv.foreground=WhitePixel(X_ecran, DefaultScreen(X_ecran));
  gcv.background=BlackPixel(X_ecran, DefaultScreen(X_ecran));
  gcv.line_width=1;
  gcv.function=GXcopy;
  gcv.plane_mask=AllPlanes;
  gcv.fill_style = FillSolid;
  gcv.join_style = JoinMiter; /* JoinMiter, JoinRound, JoinBevel */
  gcv.fill_rule = EvenOddRule; /* ou WindingRule */
  gcv.arc_mode = ArcChord; /* ou ArcPieSlice */
  gcontext=XCreateGC(X_ecran,X_win,GCFillRule    |
		                   GCLineWidth   |
		                   GCArcMode     |
                                   GCFillStyle   |
		                   GCFunction    |
		                   GCForeground  |
		                   GCBackground  |
		                   GCPlaneMask   ,
		     &gcv);
  X_font=XLoadQueryFont(X_ecran, FONT);
  XMapWindow (X_ecran, X_win);
  XStoreName(X_ecran,X_win,"Casse - tte");
  XFlush (X_ecran);
}

void X_close_screen(void)
{
  XFreeGC(X_ecran, gcontext);
  XDestroyWindow(X_ecran, X_win);
  XCloseDisplay (X_ecran);
}

void X_clear_screen(void)
{
  XClearWindow(X_ecran, X_win);
}

#include <math.h>

void X_put_fond(void)
{
  int rayon_cercle = ECRAN_HAUT/6;
  int sq=(int)sqrt((double)(2*rayon_cercle*rayon_cercle));
  int cx=(sq+1)*(6+7-6)-rayon_cercle, cy=ECRAN_HAUT/2-rayon_cercle;
  XFillRectangle(X_ecran, X_win, gcontext, 0, 0, ECRAN_LARG, ECRAN_HAUT/3);
  XFillRectangle(X_ecran, X_win, gcontext, 0, 2*ECRAN_HAUT/3-rayon_cercle/4, ECRAN_LARG, ECRAN_HAUT/3);
  XFillRectangle(X_ecran, X_win, gcontext, (sq+1)*(8+7-6)-rayon_cercle/2, cy-1,
		 ECRAN_LARG, rayon_cercle/4);
  gcv.foreground=BlackPixel(X_ecran, DefaultScreen(X_ecran));
  gcv.background=WhitePixel(X_ecran, DefaultScreen(X_ecran));
  gcv.arc_mode = ArcChord;
  XChangeGC (X_ecran, gcontext, GCArcMode|GCForeground|GCBackground, &gcv);
  XFillArc(X_ecran, X_win, gcontext, cx-3, cy-3,
	   2*rayon_cercle+5, 2*rayon_cercle+5, -180*64, 180*64);
  gcv.foreground=WhitePixel(X_ecran, DefaultScreen(X_ecran));
  gcv.background=BlackPixel(X_ecran, DefaultScreen(X_ecran));
  gcv.arc_mode = ArcPieSlice;
  if(regarde_gagne()) X_put_gagne();
  XChangeGC (X_ecran, gcontext, GCArcMode|GCForeground|GCBackground, &gcv);
}

#include <string.h>

void X_put_gagne(void)
{
  XTextItem txt={0};
  char msg[]="jeu gagne";
  int x,y;

  msg[8]=0xE9; /* e acute */
  txt.chars=msg;
  txt.nchars=strlen(msg);
  txt.font=X_font->fid;
  x=ECRAN_LARG/2;
  y=20;
  XDrawText(X_ecran,X_win,gcontext,x,y,&txt,1);
}

void _X_put (int pos, trou_pos rot, int special)
{
  int rayon_cercle = ECRAN_HAUT/6;
  int sq=(int)sqrt((double)(2*rayon_cercle*rayon_cercle));
  int cx=(sq+1)*pos,
      cy=ECRAN_HAUT/2;
  int angle1=0, angle2, angle3, angle4;
  int c1x=0, c1y=0, c2x=0, c2y=0, c3x=0, c3y=0, c4x=0, c4y=0;
  switch (rot) {
    case TROU_DROIT  :
      c1x=cx, c1y= cy;
      c2x=cx, c2y= cy - sq;
      c3x=cx - sq, c3y= cy;
      c4x=cx, c4y = cy+ sq;
      angle1=-64*45;
      break;
    case TROU_GAUCHE :
      c1x=cx, c1y= cy;
      c2x=cx, c2y= cy + sq;
      c3x=cx + sq, c3y= cy;
      c4x=cx, c4y = cy- sq;
      angle1=64*45*3;
      break;
    case TROU_HAUT   :
      c1x=cx, c1y= cy;
      c2x=cx-sq, c2y= cy;
      c3x=cx, c3y= cy+sq;
      c4x=cx+sq, c4y = cy;
      angle1=64*45;
      break;
    case TROU_BAS    :
      c1x=cx, c1y= cy;
      c2x=cx+sq, c2y= cy;
      c3x=cx, c3y= cy-sq;
      c4x=cx-sq, c4y = cy;
      angle1=64*45*5;
  }
  angle2=angle1+90*64;  angle3=angle2+90*64;  angle4=angle3+90*64;

  XFillRectangle(X_ecran, X_win, gcontext, cx-sq/2+1, cy-sq/2+1, sq-1, sq-1);
  XFillArc(X_ecran, X_win, gcontext, c1x-rayon_cercle, c1y-rayon_cercle,
	   2*rayon_cercle, 2*rayon_cercle, angle1, 90*64);
  gcv.foreground=BlackPixel(X_ecran, DefaultScreen(X_ecran));
  gcv.background=WhitePixel(X_ecran, DefaultScreen(X_ecran));
  gcv.arc_mode = ArcChord;
  XChangeGC (X_ecran, gcontext, GCArcMode|GCForeground|GCBackground, &gcv);
  XFillArc(X_ecran, X_win, gcontext, c2x-rayon_cercle, c2y-rayon_cercle,
	   2*rayon_cercle, 2*rayon_cercle, angle4, 90*64);
  if (!special)
    XFillArc(X_ecran, X_win, gcontext, c3x-rayon_cercle, c3y-rayon_cercle,
	     2*rayon_cercle, 2*rayon_cercle, angle1, 90*64);
  XFillArc(X_ecran, X_win, gcontext, c4x-rayon_cercle, c4y-rayon_cercle,
	   2*rayon_cercle, 2*rayon_cercle, angle2, 90*64);
  gcv.foreground=WhitePixel(X_ecran, DefaultScreen(X_ecran));
  gcv.background=BlackPixel(X_ecran, DefaultScreen(X_ecran));
  gcv.arc_mode = ArcPieSlice;
  XChangeGC (X_ecran, gcontext, GCArcMode|GCForeground|GCBackground, &gcv);
}

void X_put_chose(int pos, trou_pos rot)
{
  _X_put (pos, rot, 0);
}

void X_put_derniere_chose(int pos, trou_pos rot)
{
  _X_put (pos, rot, 1);
}
